/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.servicemix.eip.patterns;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.dom.DOMSource;

import org.apache.servicemix.expression.PropertyExpression;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class RecipientListAggregator extends SplitAggregator {

	public RecipientListAggregator() {
		super();
		this.count = new PropertyExpression(
				StaticRecipientList.RECIPIENT_LIST_COUNT);
		this.index = new PropertyExpression(
				StaticRecipientList.RECIPIENT_LIST_INDEX);
		this.corrId = new PropertyExpression(
				StaticRecipientList.RECIPIENT_LIST_CORRID);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.servicemix.eip.support.Aggregation#buildAggregate(javax.jbi.messaging.NormalizedMessage,
	 *      javax.jbi.messaging.MessageExchange, boolean)
	 */
	public void buildAggregate(Object aggregation, NormalizedMessage message,
			MessageExchange exchange, boolean timeout) throws Exception {
		NormalizedMessage[] messages = ((SplitterAggregation) aggregation).messages;
		String correlationId = ((SplitterAggregation) aggregation).correlationId;
		SourceTransformer st = new SourceTransformer();
		Document doc = st.createDocument();
		Element root = createChildElement(getAggregateElementName(), doc);
		root.setAttribute(getCountAttribute(), Integer
				.toString(messages.length));
		for (int i = 0; i < messages.length; i++) {
			if (messages[i] != null) {
				Element elem = st.toDOMElement(messages[i]);
				if (getMessageElementName() != null) {
					Element msg = createChildElement(getMessageElementName(),
							root);
					msg.setAttribute(getIndexAttribute(), Integer.toString(i));
					msg.appendChild(doc.importNode(elem, true));
				} else {
					root.appendChild(doc.importNode(elem, true));
				}
			}
		}
		message.setContent(new DOMSource(doc));
		message.setProperty(StaticRecipientList.RECIPIENT_LIST_CORRID,
				correlationId);
	}

}
